@@ -1,4 +1,3 @@
-import json
 from typing import Union, List, Tuple
 
 
@@ -13,11 +12,7 @@ class Path:
         return res
 
     def serialize(self) -> dict:
-        return {
-            "route": [
-                {"x": i[0], "y": i[1]} for i in self.route
-            ]
-        }
+        return {"route": [{"x": i[0], "y": i[1]} for i in self.route]}
 
 
 class Solution:
@@ -34,8 +29,4 @@ class Solution:
         self.paths.append(path)
 
     def serialize(self) -> dict:
-        return {
-            "paths": [
-                i.serialize() for i in self.paths
-            ]
-        }
+        return {"paths": [i.serialize() for i in self.paths]}
